# Copyright (c) MONAI Consortium
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import unittest

import numpy as np
import torch
from parameterized import parameterized

from monai.data import MetaTensor, set_track_meta
from monai.transforms import RandFlipd
from tests.lazy_transforms_utils import test_resampler_lazy
from tests.utils import TEST_NDARRAYS_ALL, NumpyImageTestCase2D, assert_allclose, test_local_inversion

VALID_CASES = [("no_axis", None), ("one_axis", 1), ("many_axis", [0, 1])]


class TestRandFlipd(NumpyImageTestCase2D):
    @parameterized.expand(VALID_CASES)
    def test_correct_results(self, _, spatial_axis):
        for p in TEST_NDARRAYS_ALL:
            init_param = {"keys": "img", "prob": 1.0, "spatial_axis": spatial_axis}
            flip = RandFlipd(**init_param)
            im = p(self.imt[0])
            call_param = {"data": {"img": im}}
            result = flip(**call_param)

            # test lazy
            test_resampler_lazy(flip, result, init_param, call_param, output_key="img")
            flip.lazy = False

            expected = [np.flip(channel, spatial_axis) for channel in self.imt[0]]
            expected = np.stack(expected)
            assert_allclose(result["img"], p(expected), type_test="tensor")
            test_local_inversion(flip, {"img": result["img"]}, {"img": im}, "img")

            set_track_meta(False)
            result = flip({"img": im})["img"]
            self.assertNotIsInstance(result, MetaTensor)
            self.assertIsInstance(result, torch.Tensor)
            set_track_meta(True)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) MONAI Consortium
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import unittest

from parameterized import parameterized

from monai.bundle import get_all_bundles_list, get_bundle_info, get_bundle_versions
from monai.utils import optional_import
from tests.utils import SkipIfNoModule, skip_if_downloading_fails, skip_if_quick, skip_if_windows

requests, _ = optional_import("requests")

TEST_CASE_1 = [{"bundle_name": "brats_mri_segmentation"}]

TEST_CASE_2 = [{"bundle_name": "spleen_ct_segmentation", "version": "0.1.0", "auth_token": None}]

TEST_CASE_FAKE_TOKEN = [{"bundle_name": "spleen_ct_segmentation", "version": "0.1.0", "auth_token": "ghp_errortoken"}]


@skip_if_windows
@SkipIfNoModule("requests")
class TestGetBundleData(unittest.TestCase):
    @skip_if_quick
    def test_get_all_bundles_list(self):
        with skip_if_downloading_fails():
            output = get_all_bundles_list()
            self.assertTrue(isinstance(output, list))
            self.assertTrue(isinstance(output[0], tuple))
            self.assertTrue(len(output[0]) == 2)

    @parameterized.expand([TEST_CASE_1])
    @skip_if_quick
    def test_get_bundle_versions(self, params):
        with skip_if_downloading_fails():
            output = get_bundle_versions(**params)
            self.assertTrue(isinstance(output, dict))
            self.assertTrue("latest_version" in output and "all_versions" in output)
            self.assertTrue("0.1.0" in output["all_versions"])

    @parameterized.expand([TEST_CASE_1, TEST_CASE_2])
    @skip_if_quick
    def test_get_bundle_info(self, params):
        with skip_if_downloading_fails():
            output = get_bundle_info(**params)
            self.assertTrue(isinstance(output, dict))
            for key in ["id", "name", "size", "download_count", "browser_download_url"]:
                self.assertTrue(key in output)

    @parameterized.expand([TEST_CASE_FAKE_TOKEN])
    @skip_if_quick
    def test_fake_token(self, params):
        with skip_if_downloading_fails():
            with self.assertRaises(requests.exceptions.HTTPError):
                get_bundle_info(**params)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) MONAI Consortium
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import unittest

from parameterized import parameterized

from monai.transforms import SpatialCropd
from tests.croppers import CropTest

TESTS = [
    [
        {"keys": ["img"], "roi_center": [1, 1], "roi_size": [2, 2]},
        (1, 3, 3),
        (1, 2, 2),
        (slice(None), slice(None, 2), slice(None, 2)),
    ],
    [
        {"keys": ["img"], "roi_center": [1, 1, 1], "roi_size": [2, 2, 2]},
        (3, 3, 3, 3),
        (3, 2, 2, 2),
        (slice(None), slice(None, 2), slice(None, 2), slice(None, 2)),
    ],
    [
        {"keys": ["img"], "roi_start": [0, 0, 0], "roi_end": [2, 2, 2]},
        (3, 3, 3, 3),
        (3, 2, 2, 2),
        (slice(None), slice(None, 2), slice(None, 2), slice(None, 2)),
    ],
    [
        {"keys": ["img"], "roi_start": [0, 0], "roi_end": [2, 2]},
        (3, 3, 3, 3),
        (3, 2, 2, 3),
        (slice(None), slice(None, 2), slice(None, 2), slice(None)),
    ],
    [
        {"keys": ["img"], "roi_start": [0, 0, 0, 0, 0], "roi_end": [2, 2, 2, 2, 2]},
        (3, 3, 3, 3),
        (3, 2, 2, 2),
        (slice(None), slice(None, 2), slice(None, 2), slice(None, 2)),
    ],
    [
        {"keys": ["img"], "roi_slices": [slice(s, e) for s, e in zip([-1, -2, 0], [None, None, 2])]},
        (3, 3, 3, 3),
        (3, 1, 2, 2),
        (slice(None), slice(-1, None), slice(-2, None), slice(0, 2)),
    ],
]


class TestSpatialCropd(CropTest):
    Cropper = SpatialCropd

    @parameterized.expand(TESTS)
    def test_shape(self, input_param, input_shape, expected_shape, same_area):
        self.crop_test(input_param, input_shape, expected_shape, same_area)

    @parameterized.expand(TESTS)
    def test_pending_ops(self, input_param, input_shape, _expected_shape, _same_area):
        self.crop_test_pending_ops(input_param, input_shape)


if __name__ == "__main__":
    unittest.main()
